@@ -78,8 +78,8 @@ void AdBlockManager::setEnabled(bool enabled)
     emit enabledChanged(enabled);
 
     Settings settings;
-    settings.beginGroup("AdBlock");
-    settings.setValue("enabled", m_enabled);
+    settings.beginGroup(QSL("AdBlock"));
+    settings.setValue(QSL("enabled"), m_enabled);
     settings.endGroup();
 
     load();
@@ -172,9 +172,9 @@ bool AdBlockManager::addSubscriptionFromUrl(const QUrl &url)
 
     for (int i = 0; i < queryItems.count(); ++i) {
         QPair<QString, QString> pair = queryItems.at(i);
-        if (pair.first == QL1S("location"))
+        if (pair.first.endsWith(QL1S("location")))
             subscriptionUrl = pair.second;
-        else if (pair.first == QL1S("title"))
+        else if (pair.first.endsWith(QL1S("title")))
             subscriptionTitle = pair.second;
     }
 
@@ -183,7 +183,7 @@ bool AdBlockManager::addSubscriptionFromUrl(const QUrl &url)
 
     const QString message = AdBlockManager::tr("Do you want to add <b>%1</b> subscription?").arg(subscriptionTitle);
 
-    QMessageBox::StandardButton result = QMessageBox::question(0, AdBlockManager::tr("AdBlock Subscription"), message, QMessageBox::Yes | QMessageBox::No);
+    QMessageBox::StandardButton result = QMessageBox::question(nullptr, AdBlockManager::tr("AdBlock Subscription"), message, QMessageBox::Yes | QMessageBox::No);
     if (result == QMessageBox::Yes) {
         AdBlockManager::instance()->addSubscription(subscriptionTitle, subscriptionUrl);
         AdBlockManager::instance()->showDialog();
@@ -195,18 +195,18 @@ bool AdBlockManager::addSubscriptionFromUrl(const QUrl &url)
 AdBlockSubscription* AdBlockManager::addSubscription(const QString &title, const QString &url)
 {
     if (title.isEmpty() || url.isEmpty()) {
-        return 0;
+        return nullptr;
     }
 
-    QString fileName = QzTools::filterCharsFromFilename(title.toLower()) + ".txt";
-    QString filePath = QzTools::ensureUniqueFilename(DataPaths::currentProfilePath() + "/adblock/" + fileName);
+    QString fileName = QzTools::filterCharsFromFilename(title.toLower()) + QSL(".txt");
+    QString filePath = QzTools::ensureUniqueFilename(DataPaths::currentProfilePath() + QSL("/adblock/") + fileName);
 
-    QByteArray data = QString("Title: %1\nUrl: %2\n[Adblock Plus 1.1.1]").arg(title, url).toLatin1();
+    QByteArray data = QSL("Title: %1\nUrl: %2\n[Adblock Plus 1.1.1]").arg(title, url).toLatin1();
 
     QSaveFile file(filePath);
     if (!file.open(QFile::WriteOnly)) {
         qWarning() << "AdBlockManager: Cannot write to file" << filePath;
-        return 0;
+        return nullptr;
     }
     file.write(data);
     file.commit();
@@ -217,8 +217,8 @@ AdBlockSubscription* AdBlockManager::addSubscription(const QString &title, const
     subscription->loadSubscription(m_disabledRules);
 
     m_subscriptions.insert(m_subscriptions.count() - 1, subscription);
-    connect(subscription, SIGNAL(subscriptionUpdated()), mApp, SLOT(reloadUserStyleSheet()));
-    connect(subscription, SIGNAL(subscriptionChanged()), this, SLOT(updateMatcher()));
+    connect(subscription, &AdBlockSubscription::subscriptionUpdated, mApp, &MainApplication::reloadUserStyleSheet);
+    connect(subscription, &AdBlockSubscription::subscriptionChanged, this, &AdBlockManager::updateMatcher);
 
     return subscription;
 }
@@ -242,7 +242,7 @@ bool AdBlockManager::removeSubscription(AdBlockSubscription* subscription)
 
 AdBlockCustomList* AdBlockManager::customList() const
 {
-    foreach (AdBlockSubscription* subscription, m_subscriptions) {
+    for (AdBlockSubscription* subscription : qAsConst(m_subscriptions)) {
         AdBlockCustomList* list = qobject_cast<AdBlockCustomList*>(subscription);
 
         if (list) {
@@ -250,7 +250,7 @@ AdBlockCustomList* AdBlockManager::customList() const
         }
     }
 
-    return 0;
+    return nullptr;
 }
 
 void AdBlockManager::load()
@@ -267,23 +267,24 @@ void AdBlockManager::load()
 #endif
 
     Settings settings;
-    settings.beginGroup("AdBlock");
-    m_enabled = settings.value("enabled", m_enabled).toBool();
-    m_disabledRules = settings.value("disabledRules", QStringList()).toStringList();
-    QDateTime lastUpdate = settings.value("lastUpdate", QDateTime()).toDateTime();
+    settings.beginGroup(QSL("AdBlock"));
+    m_enabled = settings.value(QSL("enabled"), m_enabled).toBool();
+    m_disabledRules = settings.value(QSL("disabledRules"), QStringList()).toStringList();
+    QDateTime lastUpdate = settings.value(QSL("lastUpdate"), QDateTime()).toDateTime();
     settings.endGroup();
 
     if (!m_enabled) {
         return;
     }
 
-    QDir adblockDir(DataPaths::currentProfilePath() + "/adblock");
-    // Create if neccessary
+    QDir adblockDir(DataPaths::currentProfilePath() + QSL("/adblock"));
+    // Create if necessary
     if (!adblockDir.exists()) {
-        QDir(DataPaths::currentProfilePath()).mkdir("adblock");
+        QDir(DataPaths::currentProfilePath()).mkdir(QSL("adblock"));
     }
 
-    foreach (const QString &fileName, adblockDir.entryList(QStringList("*.txt"), QDir::Files)) {
+    const auto fileNames = adblockDir.entryList(QStringList(QSL("*.txt")), QDir::Files);
+    for (const QString &fileName : fileNames) {
         if (fileName == QLatin1String("customlist.txt")) {
             continue;
         }
@@ -329,15 +330,15 @@ void AdBlockManager::load()
     m_subscriptions.append(customList);
 
     // Load all subscriptions
-    foreach (AdBlockSubscription* subscription, m_subscriptions) {
+    for (AdBlockSubscription* subscription : qAsConst(m_subscriptions)) {
         subscription->loadSubscription(m_disabledRules);
 
-        connect(subscription, SIGNAL(subscriptionUpdated()), mApp, SLOT(reloadUserStyleSheet()));
-        connect(subscription, SIGNAL(subscriptionChanged()), this, SLOT(updateMatcher()));
+        connect(subscription, &AdBlockSubscription::subscriptionUpdated, mApp, &MainApplication::reloadUserStyleSheet);
+        connect(subscription, &AdBlockSubscription::subscriptionChanged, this, &AdBlockManager::updateMatcher);
     }
 
     if (lastUpdate.addDays(5) < QDateTime::currentDateTime()) {
-        QTimer::singleShot(1000 * 60, this, SLOT(updateAllSubscriptions()));
+        QTimer::singleShot(1000 * 60, this, &AdBlockManager::updateAllSubscriptions);
     }
 
 #ifdef ADBLOCK_DEBUG
@@ -359,18 +360,20 @@ void AdBlockManager::updateMatcher()
 {
     QMutexLocker locker(&m_mutex);
 
+    mApp->networkManager()->removeUrlInterceptor(m_interceptor);
     m_matcher->update();
+    mApp->networkManager()->installUrlInterceptor(m_interceptor);
 }
 
 void AdBlockManager::updateAllSubscriptions()
 {
-    foreach (AdBlockSubscription* subscription, m_subscriptions) {
+    for (AdBlockSubscription* subscription : qAsConst(m_subscriptions)) {
         subscription->updateSubscription();
     }
 
     Settings settings;
-    settings.beginGroup("AdBlock");
-    settings.setValue("lastUpdate", QDateTime::currentDateTime());
+    settings.beginGroup(QSL("AdBlock"));
+    settings.setValue(QSL("lastUpdate"), QDateTime::currentDateTime());
     settings.endGroup();
 }
 
@@ -380,14 +383,14 @@ void AdBlockManager::save()
         return;
     }
 
-    foreach (AdBlockSubscription* subscription, m_subscriptions) {
+    for (AdBlockSubscription* subscription : qAsConst(m_subscriptions)) {
         subscription->saveSubscription();
     }
 
     Settings settings;
-    settings.beginGroup("AdBlock");
-    settings.setValue("enabled", m_enabled);
-    settings.setValue("disabledRules", m_disabledRules);
+    settings.beginGroup(QSL("AdBlock"));
+    settings.setValue(QSL("enabled"), m_enabled);
+    settings.setValue(QSL("disabledRules"), m_disabledRules);
     settings.endGroup();
 }
 
@@ -409,7 +412,7 @@ bool AdBlockManager::canBeBlocked(const QUrl &url) const
 
 QString AdBlockManager::elementHidingRules(const QUrl &url) const
 {
-    if (!isEnabled() || !canRunOnScheme(url.scheme()) || !canBeBlocked(url))
+    if (!isEnabled() || !canRunOnScheme(url.scheme()) || m_matcher->genericElemHideDisabledForUrl(url))
         return QString();
 
     return m_matcher->elementHidingRules();
@@ -417,7 +420,7 @@ QString AdBlockManager::elementHidingRules(const QUrl &url) const
 
 QString AdBlockManager::elementHidingRulesForDomain(const QUrl &url) const
 {
-    if (!isEnabled() || !canRunOnScheme(url.scheme()) || !canBeBlocked(url))
+    if (!isEnabled() || !canRunOnScheme(url.scheme()) || m_matcher->elemHideDisabledForUrl(url))
         return QString();
 
     return m_matcher->elementHidingRulesForDomain(url.host());
@@ -425,19 +428,19 @@ QString AdBlockManager::elementHidingRulesForDomain(const QUrl &url) const
 
 AdBlockSubscription* AdBlockManager::subscriptionByName(const QString &name) const
 {
-    foreach (AdBlockSubscription* subscription, m_subscriptions) {
+    for (AdBlockSubscription* subscription : qAsConst(m_subscriptions)) {
         if (subscription->title() == name) {
             return subscription;
         }
     }
 
-    return 0;
+    return nullptr;
 }
 
-AdBlockDialog* AdBlockManager::showDialog()
+AdBlockDialog *AdBlockManager::showDialog(QWidget *parent)
 {
     if (!m_adBlockDialog) {
-        m_adBlockDialog = new AdBlockDialog(mApp->getWindow());
+        m_adBlockDialog = new AdBlockDialog(parent ? parent : mApp->getWindow());
     }
 
     m_adBlockDialog.data()->show();
