@@ -40,6 +40,7 @@
 #include <QStandardPaths>
 #include <QWebEngineHistory>
 #include <QWebEngineDownloadItem>
+#include <QtWebEngineWidgetsVersion>
 
 #ifdef Q_OS_WIN
 #include <QtWin>
@@ -64,10 +65,10 @@ DownloadManager::DownloadManager(QWidget* parent)
     ui->clearButton->setIcon(QIcon::fromTheme("edit-clear"));
     QzTools::centerWidgetOnScreen(this);
 
-    connect(ui->clearButton, SIGNAL(clicked()), this, SLOT(clearList()));
+    connect(ui->clearButton, &QAbstractButton::clicked, this, &DownloadManager::clearList);
 
     QShortcut* clearShortcut = new QShortcut(QKeySequence("CTRL+L"), this);
-    connect(clearShortcut, SIGNAL(activated()), this, SLOT(clearList()));
+    connect(clearShortcut, &QShortcut::activated, this, &DownloadManager::clearList);
 
     loadSettings();
 
@@ -118,7 +119,7 @@ void DownloadManager::keyPressEvent(QKeyEvent* e)
     QWidget::keyPressEvent(e);
 }
 
-void DownloadManager::closeDownloadTab(const QUrl &url) const
+void DownloadManager::closeDownloadTab(QWebEngineDownloadItem *item) const
 {
     // Attempt to close empty tab that was opened only for loading the download url
     auto testWebView = [](TabbedWebView *view, const QUrl &url) {
@@ -132,6 +133,10 @@ void DownloadManager::closeDownloadTab(const QUrl &url) const
         if (page->history()->count() != 0) {
             return false;
         }
+#if QTWEBENGINEWIDGETS_VERSION >= QT_VERSION_CHECK(5, 12, 0)
+        Q_UNUSED(url)
+        return true;
+#else
         if (page->url() != QUrl()) {
             return false;
         }
@@ -140,10 +145,33 @@ void DownloadManager::closeDownloadTab(const QUrl &url) const
             tabUrl = QUrl(view->webTab()->locationBar()->text());
         }
         return tabUrl.host() == url.host();
+#endif
     };
 
-    if (testWebView(mApp->getWindow()->weView(), url)) {
-        mApp->getWindow()->weView()->closeView();
+#if QTWEBENGINEWIDGETS_VERSION >= QT_VERSION_CHECK(5, 12, 0)
+    if (!item->page()) {
+        return;
+    }
+    WebPage *page = qobject_cast<WebPage*>(item->page());
+    if (!page) {
+        return;
+    }
+    TabbedWebView *view = qobject_cast<TabbedWebView*>(page->view());
+    if (!view) {
+        return;
+    }
+    if (testWebView(view, QUrl())) {
+        view->closeView();
+    }
+#else
+    BrowserWindow* mainWindow = mApp->getWindow();
+    // If the main window was closed, there is no need to go further
+    if (mainWindow == nullptr) {
+        return;
+    }
+
+    if (testWebView(mainWindow->weView(), item->url())) {
+        mainWindow->weView()->closeView();
         return;
     }
 
@@ -152,12 +180,13 @@ void DownloadManager::closeDownloadTab(const QUrl &url) const
         const auto tabs = window->tabWidget()->allTabs();
         for (auto *tab : tabs) {
             auto *view = tab->webView();
-            if (testWebView(view, url)) {
+            if (testWebView(view, item->url())) {
                 view->closeView();
                 return;
             }
         }
     }
+#endif
 }
 
 QWinTaskbarButton *DownloadManager::taskbarButton()
@@ -212,20 +241,20 @@ void DownloadManager::timerEvent(QTimerEvent* e)
         }
 
         QTime remaining;
-        foreach (const QTime &time, remTimes) {
+        for (const QTime &time : qAsConst(remTimes)) {
             if (time > remaining) {
                 remaining = time;
             }
         }
 
         int progress = 0;
-        foreach (int prog, progresses) {
+        for (int prog : qAsConst(progresses)) {
             progress += prog;
         }
         progress = progress / progresses.count();
 
         double speed = 0.00;
-        foreach (double spee, speeds) {
+        for (double spee : qAsConst(speeds)) {
             speed += spee;
         }
 
@@ -263,15 +292,15 @@ void DownloadManager::clearList()
 
 void DownloadManager::download(QWebEngineDownloadItem *downloadItem)
 {
-    closeDownloadTab(downloadItem->url());
+    QTime downloadTimer;
+    downloadTimer.start();
+
+    closeDownloadTab(downloadItem);
 
     QString downloadPath;
     bool openFile = false;
 
-    QString fileName = QFileInfo(downloadItem->path()).fileName();
-    fileName = QUrl::fromPercentEncoding(fileName.toUtf8());
-    // Filename may have been percent encoded and actually containing path
-    fileName = QFileInfo(fileName).fileName();
+    const QString fileName = QFileInfo(downloadItem->path()).fileName();
 
     const bool forceAsk = downloadItem->savePageFormat() != QWebEngineDownloadItem::UnknownSaveFormat
             || downloadItem->type() == QWebEngineDownloadItem::UserRequested;
@@ -370,8 +399,10 @@ void DownloadManager::download(QWebEngineDownloadItem *downloadItem)
     // Create download item
     QListWidgetItem* listItem = new QListWidgetItem(ui->list);
     DownloadItem* downItem = new DownloadItem(listItem, downloadItem, QFileInfo(downloadPath).absolutePath(), QFileInfo(downloadPath).fileName(), openFile, this);
-    connect(downItem, SIGNAL(deleteItem(DownloadItem*)), this, SLOT(deleteItem(DownloadItem*)));
-    connect(downItem, SIGNAL(downloadFinished(bool)), this, SLOT(downloadFinished(bool)));
+    downItem->setDownTimer(downloadTimer);
+    downItem->startDownloading();
+    connect(downItem, &DownloadItem::deleteItem, this, &DownloadManager::deleteItem);
+    connect(downItem, &DownloadItem::downloadFinished, this, &DownloadManager::downloadFinished);
     ui->list->setItemWidget(listItem, downItem);
     listItem->setSizeHint(downItem->sizeHint());
     downItem->show();
