@@ -50,6 +50,7 @@
 #include <QPrintDialog>
 #include <QPrinter>
 #include <QQuickWidget>
+#include <QtWebEngineWidgetsVersion>
 
 bool WebView::s_forceContextMenuOnMouseRelease = false;
 
@@ -495,7 +496,7 @@ void WebView::copyLinkToClipboard()
 
 void WebView::savePageAs()
 {
-#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
+#if QTWEBENGINEWIDGETS_VERSION >= QT_VERSION_CHECK(5, 10, 0)
     page()->runJavaScript(QSL("document.contentType"), WebPage::SafeJsWorld, [this](const QVariant &res) {
         const QSet<QString> webPageTypes = {
             QSL("text/html"),
@@ -752,10 +753,10 @@ void WebView::createPageContextMenu(QMenu* menu)
     // Special menu for Speed Dial page
     if (url().toString() == QL1S("falkon:speeddial")) {
         menu->addSeparator();
-        menu->addAction(QIcon::fromTheme("list-add"), tr("&Add New Page"), this, SLOT(addSpeedDial()));
-        menu->addAction(IconProvider::settingsIcon(), tr("&Configure Speed Dial"), this, SLOT(configureSpeedDial()));
+        menu->addAction(QIcon::fromTheme("list-add"), tr("&Add New Page"), this, &WebView::addSpeedDial);
+        menu->addAction(IconProvider::settingsIcon(), tr("&Configure Speed Dial"), this, &WebView::configureSpeedDial);
         menu->addSeparator();
-        menu->addAction(QIcon::fromTheme(QSL("view-refresh")), tr("Reload All Dials"), this, SLOT(reloadAllSpeedDials()));
+        menu->addAction(QIcon::fromTheme(QSL("view-refresh")), tr("Reload All Dials"), this, &WebView::reloadAllSpeedDials);
         return;
     }
 
@@ -774,28 +775,22 @@ void WebView::createPageContextMenu(QMenu* menu)
     });
 
     menu->addSeparator();
-    menu->addAction(QIcon::fromTheme("bookmark-new"), tr("Book&mark page"), this, SLOT(bookmarkLink()));
-    menu->addAction(QIcon::fromTheme("document-save"), tr("&Save page as..."), this, SLOT(savePageAs()));
-    menu->addAction(QIcon::fromTheme("edit-copy"), tr("&Copy page link"), this, SLOT(copyLinkToClipboard()))->setData(url());
-    menu->addAction(QIcon::fromTheme("mail-message-new"), tr("Send page link..."), this, SLOT(sendPageByMail()));
+    menu->addAction(QIcon::fromTheme("bookmark-new"), tr("Book&mark page"), this, &WebView::bookmarkLink);
+    menu->addAction(QIcon::fromTheme("document-save"), tr("&Save page as..."), this, &WebView::savePageAs);
+    menu->addAction(QIcon::fromTheme("edit-copy"), tr("&Copy page link"), this, &WebView::copyLinkToClipboard)->setData(url());
+    menu->addAction(QIcon::fromTheme("mail-message-new"), tr("Send page link..."), this, &WebView::sendPageByMail);
     menu->addSeparator();
-    menu->addAction(QIcon::fromTheme("edit-select-all"), tr("Select &all"), this, SLOT(editSelectAll()));
+    menu->addAction(QIcon::fromTheme("edit-select-all"), tr("Select &all"), this, &WebView::editSelectAll);
     menu->addSeparator();
 
-    if (url().scheme() == QLatin1String("http") || url().scheme() == QLatin1String("https")) {
-        const QUrl w3url = QUrl::fromEncoded("http://validator.w3.org/check?uri=" + QUrl::toPercentEncoding(url().toEncoded()));
-        menu->addAction(QIcon(":icons/sites/w3.png"), tr("Validate page"), this, SLOT(openUrlInSelectedTab()))->setData(w3url);
+    const QString scheme = url().scheme();
 
-        QByteArray langCode = QLocale::system().name().left(2).toUtf8();
-        const QUrl gturl = QUrl::fromEncoded("http://translate.google.com/translate?sl=auto&tl=" + langCode + "&u=" + QUrl::toPercentEncoding(url().toEncoded()));
-        menu->addAction(QIcon(":icons/sites/translate.png"), tr("Translate page"), this, SLOT(openUrlInSelectedTab()))->setData(gturl);
+    if (scheme != QL1S("view-source") && WebPage::internalSchemes().contains(scheme)) {
+        menu->addAction(QIcon::fromTheme("text-html"), tr("Show so&urce code"), this, &WebView::showSource);
     }
 
-    menu->addSeparator();
-    menu->addAction(QIcon::fromTheme("text-html"), tr("Show so&urce code"), this, SLOT(showSource()));
-
     if (SiteInfo::canShowSiteInfo(url()))
-        menu->addAction(QIcon::fromTheme("dialog-information"), tr("Show info ab&out site"), this, SLOT(showSiteInfo()));
+        menu->addAction(QIcon::fromTheme("dialog-information"), tr("Show info ab&out site"), this, &WebView::showSiteInfo);
 }
 
 void WebView::createLinkContextMenu(QMenu* menu, const WebHitTestResult &hitTest)
@@ -806,17 +801,17 @@ void WebView::createLinkContextMenu(QMenu* menu, const WebHitTestResult &hitTest
     connect(act, SIGNAL(triggered()), this, SLOT(userDefinedOpenUrlInNewTab()));
     connect(act, SIGNAL(ctrlTriggered()), this, SLOT(userDefinedOpenUrlInBgTab()));
     menu->addAction(act);
-    menu->addAction(IconProvider::newWindowIcon(), tr("Open link in new &window"), this, SLOT(openUrlInNewWindow()))->setData(hitTest.linkUrl());
+    menu->addAction(IconProvider::newWindowIcon(), tr("Open link in new &window"), this, &WebView::openUrlInNewWindow)->setData(hitTest.linkUrl());
     menu->addAction(IconProvider::privateBrowsingIcon(), tr("Open link in &private window"), mApp, SLOT(startPrivateBrowsing()))->setData(hitTest.linkUrl());
     menu->addSeparator();
 
     QVariantList bData;
     bData << hitTest.linkUrl() << hitTest.linkTitle();
-    menu->addAction(QIcon::fromTheme("bookmark-new"), tr("B&ookmark link"), this, SLOT(bookmarkLink()))->setData(bData);
+    menu->addAction(QIcon::fromTheme("bookmark-new"), tr("B&ookmark link"), this, &WebView::bookmarkLink)->setData(bData);
 
-    menu->addAction(QIcon::fromTheme("document-save"), tr("&Save link as..."), this, SLOT(downloadLinkToDisk()));
-    menu->addAction(QIcon::fromTheme("mail-message-new"), tr("Send link..."), this, SLOT(sendTextByMail()))->setData(hitTest.linkUrl().toEncoded());
-    menu->addAction(QIcon::fromTheme("edit-copy"), tr("&Copy link address"), this, SLOT(copyLinkToClipboard()))->setData(hitTest.linkUrl());
+    menu->addAction(QIcon::fromTheme("document-save"), tr("&Save link as..."), this, &WebView::downloadLinkToDisk);
+    menu->addAction(QIcon::fromTheme("mail-message-new"), tr("Send link..."), this, &WebView::sendTextByMail)->setData(hitTest.linkUrl().toEncoded());
+    menu->addAction(QIcon::fromTheme("edit-copy"), tr("&Copy link address"), this, &WebView::copyLinkToClipboard)->setData(hitTest.linkUrl());
     menu->addSeparator();
 
     if (!selectedText().isEmpty()) {
@@ -831,15 +826,15 @@ void WebView::createImageContextMenu(QMenu* menu, const WebHitTestResult &hitTes
     if (hitTest.imageUrl() != url()) {
         Action *act = new Action(tr("Show i&mage"));
         act->setData(hitTest.imageUrl());
-        connect(act, SIGNAL(triggered()), this, SLOT(openActionUrl()));
+        connect(act, &QAction::triggered, this, &WebView::openActionUrl);
         connect(act, SIGNAL(ctrlTriggered()), this, SLOT(userDefinedOpenUrlInNewTab()));
         menu->addAction(act);
     }
-    menu->addAction(tr("Copy image"), this, SLOT(copyImageToClipboard()));
-    menu->addAction(QIcon::fromTheme("edit-copy"), tr("Copy image ad&dress"), this, SLOT(copyLinkToClipboard()))->setData(hitTest.imageUrl());
+    menu->addAction(tr("Copy image"), this, &WebView::copyImageToClipboard);
+    menu->addAction(QIcon::fromTheme("edit-copy"), tr("Copy image ad&dress"), this, &WebView::copyLinkToClipboard)->setData(hitTest.imageUrl());
     menu->addSeparator();
-    menu->addAction(QIcon::fromTheme("document-save"), tr("&Save image as..."), this, SLOT(downloadImageToDisk()));
-    menu->addAction(QIcon::fromTheme("mail-message-new"), tr("Send image..."), this, SLOT(sendTextByMail()))->setData(hitTest.imageUrl().toEncoded());
+    menu->addAction(QIcon::fromTheme("document-save"), tr("&Save image as..."), this, &WebView::downloadImageToDisk);
+    menu->addAction(QIcon::fromTheme("mail-message-new"), tr("Send image..."), this, &WebView::sendTextByMail)->setData(hitTest.imageUrl().toEncoded());
     menu->addSeparator();
 
     if (!selectedText().isEmpty()) {
@@ -858,23 +853,9 @@ void WebView::createSelectedTextContextMenu(QMenu* menu, const WebHitTestResult
     if (!menu->actions().contains(pageAction(QWebEnginePage::Copy))) {
         menu->addAction(pageAction(QWebEnginePage::Copy));
     }
-    menu->addAction(QIcon::fromTheme("mail-message-new"), tr("Send text..."), this, SLOT(sendTextByMail()))->setData(selectedText);
+    menu->addAction(QIcon::fromTheme("mail-message-new"), tr("Send text..."), this, &WebView::sendTextByMail)->setData(selectedText);
     menu->addSeparator();
 
-    QString langCode = QLocale::system().name().left(2).toUtf8();
-    QUrl googleTranslateUrl = QUrl(QString("https://translate.google.com/#auto/%1/%2").arg(langCode, selectedText));
-    Action* gtwact = new Action(QIcon(":icons/sites/translate.png"), tr("Google Translate"));
-    gtwact->setData(googleTranslateUrl);
-    connect(gtwact, SIGNAL(triggered()), this, SLOT(openUrlInSelectedTab()));
-    connect(gtwact, SIGNAL(ctrlTriggered()), this, SLOT(openUrlInBackgroundTab()));
-    menu->addAction(gtwact);
-
-    Action* dictact = new Action(QIcon::fromTheme("accessories-dictionary"), tr("Dictionary"));
-    dictact->setData(QUrl("http://" + (!langCode.isEmpty() ? langCode + "." : langCode) + "wiktionary.org/wiki/Special:Search?search=" + selectedText));
-    connect(dictact, SIGNAL(triggered()), this, SLOT(openUrlInSelectedTab()));
-    connect(dictact, SIGNAL(ctrlTriggered()), this, SLOT(openUrlInBackgroundTab()));
-    menu->addAction(dictact);
-
     // #379: Remove newlines
     QString selectedString = selectedText.trimmed().remove(QLatin1Char('\n'));
     if (!selectedString.contains(QLatin1Char('.'))) {
@@ -887,7 +868,7 @@ void WebView::createSelectedTextContextMenu(QMenu* menu, const WebHitTestResult
         Action* act = new Action(QIcon::fromTheme("document-open-remote"), tr("Go to &web address"));
         act->setData(guessedUrl);
 
-        connect(act, SIGNAL(triggered()), this, SLOT(openActionUrl()));
+        connect(act, &QAction::triggered, this, &WebView::openActionUrl);
         connect(act, SIGNAL(ctrlTriggered()), this, SLOT(userDefinedOpenUrlInNewTab()));
         menu->addAction(act);
     }
@@ -899,20 +880,21 @@ void WebView::createSelectedTextContextMenu(QMenu* menu, const WebHitTestResult
 
     SearchEngine engine = mApp->searchEnginesManager()->defaultEngine();
     Action* act = new Action(engine.icon, tr("Search \"%1 ..\" with %2").arg(selectedText, engine.name));
-    connect(act, SIGNAL(triggered()), this, SLOT(searchSelectedText()));
-    connect(act, SIGNAL(ctrlTriggered()), this, SLOT(searchSelectedTextInBackgroundTab()));
+    connect(act, &QAction::triggered, this, &WebView::searchSelectedText);
+    connect(act, &Action::ctrlTriggered, this, &WebView::searchSelectedTextInBackgroundTab);
     menu->addAction(act);
 
     // Search with ...
     Menu* swMenu = new Menu(tr("Search with..."), menu);
     swMenu->setCloseOnMiddleClick(true);
     SearchEnginesManager* searchManager = mApp->searchEnginesManager();
-    foreach (const SearchEngine &en, searchManager->allEngines()) {
+    const auto engines = searchManager->allEngines();
+    for (const SearchEngine &en : engines) {
         Action* act = new Action(en.icon, en.name);
         act->setData(QVariant::fromValue(en));
 
-        connect(act, SIGNAL(triggered()), this, SLOT(searchSelectedText()));
-        connect(act, SIGNAL(ctrlTriggered()), this, SLOT(searchSelectedTextInBackgroundTab()));
+        connect(act, &QAction::triggered, this, &WebView::searchSelectedText);
+        connect(act, &Action::ctrlTriggered, this, &WebView::searchSelectedTextInBackgroundTab);
         swMenu->addAction(act);
     }
 
@@ -925,12 +907,12 @@ void WebView::createMediaContextMenu(QMenu *menu, const WebHitTestResult &hitTes
     bool muted = hitTest.mediaMuted();
 
     menu->addSeparator();
-    menu->addAction(paused ? tr("&Play") : tr("&Pause"), this, SLOT(toggleMediaPause()))->setIcon(QIcon::fromTheme(paused ? "media-playback-start" : "media-playback-pause"));
-    menu->addAction(muted ? tr("Un&mute") : tr("&Mute"), this, SLOT(toggleMediaMute()))->setIcon(QIcon::fromTheme(muted ? "audio-volume-muted" : "audio-volume-high"));
+    menu->addAction(paused ? tr("&Play") : tr("&Pause"), this, &WebView::toggleMediaPause)->setIcon(QIcon::fromTheme(paused ? "media-playback-start" : "media-playback-pause"));
+    menu->addAction(muted ? tr("Un&mute") : tr("&Mute"), this, &WebView::toggleMediaMute)->setIcon(QIcon::fromTheme(muted ? "audio-volume-muted" : "audio-volume-high"));
     menu->addSeparator();
-    menu->addAction(QIcon::fromTheme("edit-copy"), tr("&Copy Media Address"), this, SLOT(copyLinkToClipboard()))->setData(hitTest.mediaUrl());
-    menu->addAction(QIcon::fromTheme("mail-message-new"), tr("&Send Media Address"), this, SLOT(sendTextByMail()))->setData(hitTest.mediaUrl().toEncoded());
-    menu->addAction(QIcon::fromTheme("document-save"), tr("Save Media To &Disk"), this, SLOT(downloadMediaToDisk()));
+    menu->addAction(QIcon::fromTheme("edit-copy"), tr("&Copy Media Address"), this, &WebView::copyLinkToClipboard)->setData(hitTest.mediaUrl());
+    menu->addAction(QIcon::fromTheme("mail-message-new"), tr("&Send Media Address"), this, &WebView::sendTextByMail)->setData(hitTest.mediaUrl().toEncoded());
+    menu->addAction(QIcon::fromTheme("document-save"), tr("Save Media To &Disk"), this, &WebView::downloadMediaToDisk);
 }
 
 void WebView::checkForForm(QAction *action, const QPoint &pos)
@@ -964,17 +946,17 @@ void WebView::createSearchEngine()
 
 void WebView::addSpeedDial()
 {
-    page()->runJavaScript("addSpeedDial()");
+    page()->runJavaScript("addSpeedDial()", WebPage::SafeJsWorld);
 }
 
 void WebView::configureSpeedDial()
 {
-    page()->runJavaScript("configureSpeedDial()");
+    page()->runJavaScript("configureSpeedDial()", WebPage::SafeJsWorld);
 }
 
 void WebView::reloadAllSpeedDials()
 {
-    page()->runJavaScript("reloadAll()");
+    page()->runJavaScript("reloadAll()", WebPage::SafeJsWorld);
 }
 
 void WebView::toggleMediaPause()
@@ -1097,12 +1079,12 @@ void WebView::_mousePressEvent(QMouseEvent *event)
     }
 
     switch (event->button()) {
-    case Qt::XButton1:
+    case Qt::BackButton:
         back();
         event->accept();
         break;
 
-    case Qt::XButton2:
+    case Qt::ForwardButton:
         forward();
         event->accept();
         break;
@@ -1270,12 +1252,14 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
 
 bool WebView::focusNextPrevChild(bool next)
 {
+#if QTWEBENGINEWIDGETS_VERSION < QT_VERSION_CHECK(5, 11, 0)
     // QTBUG-67043
     // Workaround QtWebEngine issue where QWebEngineView loses focus on second load() call.
     if (next) {
         setFocus();
         return false;
     }
+#endif
     return QWebEngineView::focusNextPrevChild(next);
 }
 
@@ -1293,9 +1277,16 @@ bool WebView::eventFilter(QObject *obj, QEvent *event)
 
     // Hack to find widget that receives input events
     if (obj == this && event->type() == QEvent::ChildAdded) {
+#if QTWEBENGINEWIDGETS_VERSION >= QT_VERSION_CHECK(5, 12, 0)
+        QPointer<QWidget> child = qobject_cast<QWidget*>(static_cast<QChildEvent*>(event)->child());
+        QTimer::singleShot(0, this, [=]() {
+            if (child && child->inherits("QtWebEngineCore::RenderWidgetHostViewQtDelegateWidget")) {
+                m_rwhvqt = child;
+#else
         QTimer::singleShot(0, this, [this]() {
             if (focusProxy() && m_rwhvqt != focusProxy()) {
                 m_rwhvqt = focusProxy();
+#endif
                 m_rwhvqt->installEventFilter(this);
                 if (QQuickWidget *w = qobject_cast<QQuickWidget*>(m_rwhvqt)) {
                     w->setClearColor(palette().color(QPalette::Window));
